"use strict";

/*!
 * @wistla/util#lib/buildInfo.js
 *
 * Copyright 2017 Wistla Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var path = require('path');

var fs = require('mz/fs');
var yaml = require('js-yaml');

var CONFIG = require('config');

const { build: BUILD } = CONFIG;

let BUILD_INFO = null;

module.exports = exports = function* buildInfo(force) {
    let cache = false;
    
    try {
        BUILD_INFO = require('./build_info.json');
        
        return BUILD_INFO;
    }
    catch (e) {}
    
    try {
        let build_file_path = `${BUILD.base_path}${BUILD.file_extension}`;
        
        if (!(yield fs.stat(build_file_path)).isFile()) {
            throw new Error();
        }
        
        BUILD_INFO = yaml.safeLoad(yield fs.readFile(build_file_path, 'utf8'));
        
        if (BUILD.properties.some(property => !Object.prototype.hasOwnProperty.call(BUILD_INFO, property))) {
            throw new Error();
        }
        
        cache = true;
    }
    catch (e) {
        try {
            if (!(yield fs.stat(BUILD.base_path)).isDirectory()) {
                throw new Error();
            }
            
            let files = yield fs.readdir(BUILD.base_path);
            
            BUILD_INFO = {};
            
            yield files.map(function* (file) {
                let value = yield fs.readFile(path.resolve(BUILD.base_path, file), 'utf8');
                
                let [, key] = file.split('-');
                
                BUILD_INFO[key] = value.trim();
            });
        
            if (BUILD.properties.some(property => !Object.prototype.hasOwnProperty.call(BUILD_INFO, property))) {
                throw new Error();
            }
        
            cache = true;
        }
        catch (e) {
            let type = `BuildDataNotFoundError`;
            
            if (Object.prototype.toString.call(BUILD_INFO) === '[object Object]' && Object.keys(BUILD_INFO).length > 0) {
                BUILD_INFO.__missing__ = BUILD.properties.filter(property => !Object.prototype.hasOwnProperty.call(BUILD_INFO, property));
                
                type = `BuildDataIncompleteError; Missing => ${BUILD_INFO.__missing__.join(', ')}`;
            }
            else {
                BUILD_INFO = {
                    __error__: true
                };
            }
    
            console.trace(`@wistla/util#buildInfo ${type}`);
        }
    }
    
    if (cache === true) {
        yield fs.writeFile('./build_info.json', JSON.stringify(BUILD_INFO, null, 4), 'utf8');
    }
    
    return BUILD_INFO;
};
