"use strict";

/*!
 * @wistla/util#lib/callYieldAwait.js
 *
 * Copyright 2017 Wistla Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

const VALID_TYPES = {
    '[object Function]': true,
    '[object AsyncFunction]': true,
    '[object GeneratorFunction]': true
};

const VALID_TYPES_ENUM = '`' + Object.keys(VALID_TYPES).join('`, `') + '`';

module.exports = exports = function* callYieldAwait(fn, ...args) {
    let type = Object.prototype.toString.call(fn);
    
    if (!(type in VALID_TYPES)) {
        throw new TypeError(`@wistla/util #callOrYield expected one of ${VALID_TYPES_ENUM} and got \`${type}\` instead.`);
    }

    if (fn.FEATURE_DISABLED === true) {
        return;
    }
    
    switch (type) {
        case '[object GeneratorFunction]':
            return yield fn(...args);
            
        case '[object AsyncFunction]':
            return (async (...args) => await fn(...args))(...args);
    }
    
    return fn(...args);
};

exports.VALID_TYPES = VALID_TYPES;
exports.VALID_TYPES_ENUM = VALID_TYPES_ENUM;
