'use strict';

/*!
 * @wistla/util#lib/randomString.js
 *
 * Copyright 2017 Wistla Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

/*
 From James:
 
 Just thinking, this random function is not really as random as it could be:
 
 var rnd = crypto.pseudoRandomBytes(howMany);
 (...)
 value[i] = chars[rnd[i] % len];
 
 The problem is that there are 36 characters to choose from, which divides into 256 at 7 + 1/9. That means that the first four (36/9) characters, which are A-D, have a 1/7th higher chance of being chosen.
 
 Probably not going to keep you awake at night, but probably something we should fix, especially if we end up using this elsewhere in the future...
 */

var crypto = require('crypto');

module.exports = exports = function randomString(howMany, chars) {
    let rnd = crypto.pseudoRandomBytes(howMany);
    let value = new Array(howMany);
    let len = chars.length;
    
    for (let i = 0; i < howMany; i++) {
        value[i] = chars[rnd[i] % len];
    }
    
    return value.join('');
};
